from ftw.subsite.interfaces import ISubsiteColumn
from plone.app.portlets.manager import ColumnPortletManagerRenderer
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import adapts
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserView
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class ColumnSubsiteManagerRenderer(ColumnPortletManagerRenderer):
    """A renderer for the Subsite dashboard on the front-page,
       based on normal portlets
    """
    adapts(Interface, IDefaultBrowserLayer, IBrowserView, ISubsiteColumn)
    template = ViewPageTemplateFile('column.pt')



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.interfaces import ILanguages
from ftw.subsite.tests.base import IntegrationTestCase
from z3c.relationfield import RelationValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.intid.interfaces import IIntIds


def introduce_language_subsites(*subsites):
    intids = getUtility(IIntIds)

    for subsite in subsites:
        ids = [intids.getId(obj) for obj in subsites]
        ids.remove(intids.getId(subsite))
        subsite.language_references = [RelationValue(id_) for id_ in ids]


class TestLanguagesAdapterOnSubsite(IntegrationTestCase):

    def test_current_language_is_translated(self):
        german = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('de'))

        langs = getMultiAdapter((german, self.request), ILanguages)
        self.assertEquals(
            {'url': german.absolute_url(),
             'title': u'Deutsch',
             'code': 'de'},
            langs.get_current_language())

    def test_related_languages(self):
        german = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('de'))
        french = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('fr'))
        italian = create(Builder('subsite')
                         .titled(u'Subsite')
                         .with_language('it'))
        introduce_language_subsites(german, french, italian)

        langs = getMultiAdapter((german, self.request), ILanguages)
        self.assertEquals(
            [{'url': french.absolute_url(),
              'title': u'Fran\xe7ais',
              'code': 'fr'},

             {'url': italian.absolute_url(),
              'title': u'Italiano',
              'code': 'it'}],

            langs.get_related_languages())

    def test_related_languages_with_showing_site_root(self):
        german = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('de')
                        .having(link_site_in_languagechooser=True))
        french = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('fr')
                        .having(link_site_in_languagechooser=True))
        introduce_language_subsites(german, french)

        langs = getMultiAdapter((german, self.request), ILanguages)
        self.assertEquals(
            [{'url': self.portal.absolute_url(),
              'title': u'English',
              'code': 'en'},

             {'url': french.absolute_url(),
              'title': u'Fran\xe7ais',
              'code': 'fr'}],

            langs.get_related_languages())


class TestLanguagesAdapterOnPloneSite(IntegrationTestCase):

    def test_current_language_is_english(self):
        # The default site root language is english in the tests.
        langs = getMultiAdapter((self.portal, self.request), ILanguages)
        self.assertEquals(
            {'url': self.portal.portal_url(),
             'title': u'English',
             'code': 'en'},
            langs.get_current_language())

    def test_related_languages_lists_only_subsites_which_show_link(self):
        german = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('de')
                        .having(link_site_in_languagechooser=True))
        french = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('fr')
                        .having(link_site_in_languagechooser=True))
        create(Builder('subsite').titled(u'Subsite').with_language('it'))

        langs = getMultiAdapter((self.portal, self.request), ILanguages)
        self.assertEquals(
            [{'url': german.absolute_url(),
              'title': u'Deutsch',
              'code': 'de'},

             {'url': french.absolute_url(),
              'title': u'Fran\xe7ais',
              'code': 'fr'}],

            langs.get_related_languages())


class TestLanguagesAdapterOnSubsiteSubContent(IntegrationTestCase):

    def test_current_language_is_taken_from_next_parent_subsite(self):
        german = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('de'))
        folder = create(Builder('folder').within(german))

        langs = getMultiAdapter((folder, self.request), ILanguages)
        self.assertEquals(
            {'url': german.absolute_url(),
             'title': u'Deutsch',
             'code': 'de'},
            langs.get_current_language())

    def test_current_language_is_taken_from_parent_site_root(self):
        folder = create(Builder('folder'))

        langs = getMultiAdapter((folder, self.request), ILanguages)
        self.assertEquals(
            {'url': self.portal.portal_url(),
             'title': u'English',
             'code': 'en'},
            langs.get_current_language())

    def test_related_languages_is_inherited_from_parent_subsite(self):
        german = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('de')
                        .having(link_site_in_languagechooser=True))
        french = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('fr')
                        .having(link_site_in_languagechooser=True))
        introduce_language_subsites(german, french)

        folder = create(Builder('folder').within(german))

        langs = getMultiAdapter((folder, self.request), ILanguages)
        self.assertEquals(
            [{'url': self.portal.absolute_url(),
              'title': u'English',
              'code': 'en'},

             {'url': french.absolute_url(),
              'title': u'Fran\xe7ais',
              'code': 'fr'}],

            langs.get_related_languages())

    def test_related_languages_is_inherited_from_site_root(self):
        german = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('de')
                        .having(link_site_in_languagechooser=True))
        french = create(Builder('subsite')
                        .titled(u'Subsite')
                        .with_language('fr')
                        .having(link_site_in_languagechooser=True))

        folder = create(Builder('folder'))

        langs = getMultiAdapter((folder, self.request), ILanguages)
        self.assertEquals(
            [{'url': german.absolute_url(),
              'title': u'Deutsch',
              'code': 'de'},

             {'url': french.absolute_url(),
              'title': u'Fran\xe7ais',
              'code': 'fr'}],

            langs.get_related_languages())



from ftw.subsite.testing import FTW_SUBSITE_INTEGRATION_TESTING
from ftw.subsite.utils import find_context
from ftw.subsite.utils import get_nav_root
from plone.app.layout.navigation.interfaces import INavigationRoot
from zope.interface import alsoProvides
import unittest2 as unittest


class TestUtilsIntegration(unittest.TestCase):

    layer = FTW_SUBSITE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def tearDown(self):
        if 'folder' in self.portal.objectIds():
            self.portal.manage_delObjects(['folder'])

    def test_get_nav_root(self):
        folder = self.portal.get(self.portal.invokeFactory('Folder', 'folder'))

        self.assertEquals(self.portal, get_nav_root(folder))
        self.assertEquals(self.portal, get_nav_root(self.portal))

        doc = folder.get(folder.invokeFactory('Document', 'doc'))
        self.assertEquals(self.portal, get_nav_root(doc))

        # Create a second document, because the first call with doc is cached
        doc2 = folder.get(folder.invokeFactory('Document', 'doc2'))
        alsoProvides(folder, INavigationRoot)
        self.assertEquals(folder, get_nav_root(doc2))

    def test_find_context(self):
        # This tests the edge case if no real Request (from a browser)
        # is available. It should return the app
        # All other testcases are tested by test_forcelanguage
        doc = self.portal.get(self.portal.invokeFactory('Document', 'doc'))
        self.assertEquals(self.layer['app'], find_context(doc.REQUEST))



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.subsite.tests.helpers import introduce_language_subsites
from ftw.testbrowser import browsing
from plone.app.testing import applyProfile
from unittest2 import TestCase
import json
import transaction


class TestLanguageMobileButton(TestCase):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestLanguageMobileButton, self).setUp()
        portal = self.layer['portal']
        applyProfile(portal, 'ftw.mobile:default')
        transaction.commit()

    @browsing
    def test_shows_other_referenced_languages(self, browser):
        german = create(Builder('subsite')
                        .titled(u'Subsite DE')
                        .with_language('de'))
        french = create(Builder('subsite')
                        .titled(u'Subsite FR')
                        .with_language('fr'))
        italian = create(Builder('subsite')
                         .titled(u'Subsite IT')
                         .with_language('it'))
        introduce_language_subsites(german, french, italian)

        browser.login().visit(german)
        button = browser.css('#subsitelanguage-mobile-button a').first
        button_data = json.loads(button.attrib['data-mobile_data'])

        self.assertEquals(u'Fran\xe7ais', button_data[0]['label'])
        self.assertEquals(u'Italiano', button_data[1]['label'])

        self.assertEquals(french.absolute_url(), button_data[0]['url'])
        self.assertEquals(italian.absolute_url(), button_data[1]['url'])



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from ftw.testing import IS_PLONE_5
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import TEST_USER_ID, setRoles
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2
from zope.configuration import xmlconfig
import ftw.subsite.tests.builders  # noqa


class FtwSubsiteIntegrationLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

        # Load ZCML
        import ftw.subsite
        xmlconfig.file('configure.zcml', ftw.subsite,
                       context=configurationContext)

        xmlconfig.file('overrides.zcml', ftw.subsite,
                       context=configurationContext)

        # import plone.app.portlets
        # xmlconfig.file('configure.zcml', plone.app.portlets,
        #                context=configurationContext)

        # import collective.MockMailHost
        # xmlconfig.file('configure.zcml', collective.MockMailHost,
        #                context=configurationContext)

        # installProduct() is *only* necessary for packages outside
        # the Products.* namespace which are also declared as Zope 2 products,
        # using <five:registerPackage /> in ZCML.
        z2.installProduct(app, 'plone.app.portlets')
        z2.installProduct(app, 'ftw.subsite')
        z2.installProduct(app, 'ftw.simplelayout')
        z2.installProduct(app, 'collective.MockMailHost')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.subsite:default')
        applyProfile(portal, 'collective.MockMailHost:default')
        if IS_PLONE_5:
            applyProfile(portal, 'plone.app.contenttypes:default')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)


class FtwSubsiteWithoutApplyProfileLayer(FtwSubsiteIntegrationLayer):
    """Special layer, which does not install the ftw.subsite profile.
    This way the ftw.subsite browserlay will net be installed
    """

    def setUpPloneSite(self, portal):

        if IS_PLONE_5:
            applyProfile(portal, 'plone.app.contenttypes:default')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)


FTW_SUBSITE_FIXTURE = FtwSubsiteIntegrationLayer()
FTW_SUBSITE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_SUBSITE_FIXTURE,), name="FtwSubsite:Integration")
FTW_SUBSITE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_SUBSITE_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="FtwSubsite:Functional")

FTW_SUBSITE_SPECIAL_FIXTURE = FtwSubsiteWithoutApplyProfileLayer()
FTW_SUBSITE_SPECIAL_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FTW_SUBSITE_SPECIAL_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="FtwSubsite:SpecialFunctional")



from Acquisition import aq_inner
from Acquisition import aq_parent
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from ftw.subsite.interfaces import ILanguages
from ftw.subsite.interfaces import ISubsite
from zope.component import adapter
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.interface import Interface
from zope.interface import implementer
from zope.interface import implements


def translate_language(context, language_code):
    ltool = getToolByName(context, 'portal_languages')
    info = ltool.getAvailableLanguageInformation().get(language_code, None)
    if info is not None:
        return info.get(u'native', None)
    return None


@implementer(ILanguages)
@adapter(Interface, Interface)
def inherit_languages(context, request):
    parent = aq_parent(aq_inner(context))
    return getMultiAdapter((parent, request), ILanguages)


class SubsiteLanguages(object):
    implements(ILanguages)
    adapts(ISubsite, Interface)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_current_language(self):
        language_code = self.context.force_language
        return {'url': self.context.absolute_url(),
                'title': translate_language(self.context, language_code),
                'code': language_code}

    def get_related_languages(self):
        results = []

        for relation in self.context.language_references:
            subsite = relation.to_object
            lang_code = subsite.force_language
            if not lang_code:
                continue

            results.append({
                'url': subsite.absolute_url(),
                'title': translate_language(self.context, lang_code),
                'code': lang_code})

        if self.context.link_site_in_languagechooser:
            portal_url = getToolByName(self.context, 'portal_url')
            ltool = getToolByName(self.context, 'portal_languages')
            lang_code = ltool.getDefaultLanguage()

            results.append({
                'url': portal_url(),
                'title': translate_language(self.context, lang_code),
                'code': lang_code})

        results.sort(key=lambda item: item.get('title'))
        return results


class PloneSiteLanguages(object):
    implements(ILanguages)
    adapts(IPloneSiteRoot, Interface)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_current_language(self):
        ltool = getToolByName(self.context, 'portal_languages')
        lang_code = ltool.getDefaultLanguage()
        return {'url': self.context.absolute_url(),
                'title': translate_language(self.context, lang_code),
                'code': lang_code}

    def get_related_languages(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        results = []

        for brain in catalog(object_provides=ISubsite.__identifier__):
            subsite = brain.getObject()
            if not subsite.link_site_in_languagechooser:
                continue

            lang_code = subsite.force_language
            if not lang_code:
                continue

            results.append({
                'url': subsite.absolute_url(),
                'title': translate_language(self.context, lang_code),
                'code': lang_code})

        results.sort(key=lambda item: item.get('title'))
        return results



from ftw.mobile.buttons import BaseButton
from ftw.subsite.interfaces import ILanguages
from zope.component import getMultiAdapter


class SubsiteLanguageButton(BaseButton):

    def label(self):
        langs = getMultiAdapter((self.context, self.request), ILanguages)
        return langs.get_current_language().get('code', None)

    def position(self):
        return 300

    def data(self):
        langs = getMultiAdapter((self.context, self.request), ILanguages)

        def link_data(item):
            return {'url': item.get('url'),
                    'label': item.get('title')}
        return map(link_data, langs.get_related_languages())

    def available(self):
        langs = getMultiAdapter((self.context, self.request), ILanguages)
        return bool(langs.get_related_languages())



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.subsite.tests.helpers import LanguageSetter
from ftw.testbrowser import browsing
from unittest2 import TestCase
import transaction


class TestSubsiteForceLanguage(TestCase, LanguageSetter):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_language_tool()
        self.make_document_addable_on_subsite()

    def setup_language_tool(self):
        default = 'en'
        supported = ['en', 'de', 'fr']
        self.set_language_settings(default=default, supported=supported)

    def make_document_addable_on_subsite(self):
        types_tool = self.portal.portal_types
        subsite_fti = types_tool.get('ftw.subsite.Subsite')
        subsite_fti.allowed_content_types = ('Document', )
        transaction.commit()

    @browsing
    def test_global_lang_is_en(self, browser):
        self.assertEquals(
            'en',
            browser.login().visit().css('html').first.attrib['lang'])

    @browsing
    def test_plone_negotiator_still_works_on_non_subsite(self, browser):
        document = create(Builder('document').having(language='fr'))
        self.assertEquals(
            'fr',
            browser.login().visit(document).css('html').first.attrib['lang'])

    @browsing
    def test_global_lang_is_set_on_subsite(self, browser):
        subsite = create(Builder('subsite').having(force_language='de'))
        self.assertEquals(
            'de',
            browser.login().visit(subsite).css('html').first.attrib['lang'])

    @browsing
    def test_plone_negotiator_looses_against_subsite_language(self, browser):
        subsite = create(Builder('subsite').having(force_language='de'))
        document = create(Builder('document')
                          .within(subsite)
                          .having(language='fr'))

        self.assertEquals(
            'de',
            browser.login().visit(document).css('html').first.attrib['lang'])



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.browser.subsiteview import SubsiteView
from ftw.subsite.interfaces import IFtwSubsiteLayer
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobImage
from unittest2 import TestCase
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.viewlet.interfaces import IViewletManager
import os
import transaction


class TestLogoViewlet(TestCase):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        self.subsite = create(Builder('subsite').titled(u'MySubsite'))

    def _get_viewlet(self):
        alsoProvides(self.subsite.REQUEST, IFtwSubsiteLayer)
        view = SubsiteView(self.subsite, self.subsite.REQUEST)
        manager_name = 'plone.portalheader'
        manager = queryMultiAdapter(
            (self.subsite, self.subsite.REQUEST, view),
            IViewletManager,
            manager_name)
        self.failUnless(manager)
        # Set up viewlets
        manager.update()
        name = 'subsite.logo'
        return [v for v in manager.viewlets if v.__name__ == name]

    def _add_logo(self, subsite):
        file_ = open("%s/blue.png" % os.path.split(__file__)[0], 'r')
        file_.seek(0)
        subsite.logo = NamedBlobImage(data=file_.read(),
                                      filename=u'logo.png')
        transaction.commit()

    def assert_subsite_logo_src(self, browser):
        img_src = browser.css('#portal-logo img').first.attrib['src']
        expected_part_image_src = '{0}/@@images/'.format(
            self.subsite.absolute_url())
        assert img_src.startswith(expected_part_image_src), (
            'Wrong logo path, expect img src starting with: {0}'.format(
                expected_part_image_src))

    def test_component_registered(self):
        self.assertTrue(len(self._get_viewlet()) == 1)

    @browsing
    def test_without_logo(self, browser):
        browser.login().visit(self.subsite)

        self.assertEquals(self.subsite.absolute_url(),
                          browser.css('#portal-logo').first.attrib['href'])

        self.assertEquals('http://nohost/plone/logo.png',
                          browser.css('#portal-logo img').first.attrib['src'])

        browser.visit(self.portal)
        self.assertEquals('http://nohost/plone/logo.png',
                          browser.css('#portal-logo img').first.attrib['src'])

    @browsing
    def test_with_logo(self, browser):
        self._add_logo(self.subsite)

        browser.login().visit(self.subsite)

        self.assertEquals(self.subsite.absolute_url(),
                          browser.css('#portal-logo').first.attrib['href'])

        self.assert_subsite_logo_src(browser)

        browser.visit(self.portal)
        self.assertEquals('http://nohost/plone/logo.png',
                          browser.css('#portal-logo img').first.attrib['src'])

    @browsing
    def test_with_logo_subcontent(self, browser):
        self._add_logo(self.subsite)

        subfolder = create(Builder('folder').within(self.subsite))
        subsubfolder = create(Builder('folder').within(subfolder))

        browser.login().visit(subsubfolder)
        self.assert_subsite_logo_src(browser)

    @browsing
    def test_logo_in_portal_tools(self, browser):
        self._add_logo(self.subsite)

        browser.login().visit(self.subsite,
                              view='mail_password_form',
                              data={'userid': TEST_USER_ID})

        self.assert_subsite_logo_src(browser)

    @browsing
    def test_logo_img_tag_has_no_title_attribute(self, browser):
        self._add_logo(self.subsite)

        browser.login().visit(self.subsite)

        self.assertIsNone(
            browser.css('#portal-logo img').first.attrib.get('title', None))

    @browsing
    def test_logo_img_tag_has_no_alt_attribute_if_empty(self, browser):
        self._add_logo(self.subsite)

        browser.login().visit(self.subsite)

        self.assertIsNone(
            browser.css('#portal-logo img').first.attrib.get('alt', None))

    @browsing
    def test_logo_img_tag_has_alt_attribute_if_filled(self, browser):
        subsite = create(Builder('subsite')
                         .titled(u'My Subsite')
                         .having(logo_alt_text=u'My alt text'))
        self._add_logo(subsite)

        browser.login().visit(subsite)

        self.assertEqual(
            u'My alt text',
            browser.css('#portal-logo img').first.attrib['alt'])

        # Make sure the image has an alt text for non-subsites.
        self.portal._setProperty('logo_alt_text', 'Plone Logo Alt Text',
                                 'string')
        transaction.commit()
        browser.visit(self.portal)
        self.assertEqual(
            u'Plone Logo Alt Text',
            browser.css('#portal-logo img').first.attrib['alt'])

    @browsing
    def test_logo_with_other_content_with_id_logo(self, browser):
        self._add_logo(self.subsite)

        contentpage = create(Builder('sl content page').within(self.subsite))
        create(Builder('sl textblock').titled('logo').within(contentpage))

        browser.login().visit(contentpage)

        self.assertTrue(browser.css('#portal-logo img'))



from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from email.header import Header
from email.mime.text import MIMEText
from ftw.subsite import _
from plone.app.layout.navigation.root import getNavigationRoot
from z3c.form import form, field, button
from z3c.form.validator import SimpleFieldValidator
from z3c.form.validator import WidgetValidatorDiscriminators
from zope import schema
from zope.component import provideAdapter
from zope.i18n import translate
from zope.interface import Interface
from zope.interface import Invalid
import pkg_resources
import re


IS_PLONE_5 = pkg_resources.get_distribution('Products.CMFPlone').version >= '5'


class IContactForm(Interface):
    """Interface for z3c.form"""
    sender = schema.TextLine(title=_(u"Sender_Name", default=u"Name"),
                             required=True,
                             description=_(u"help_Sender", default="Please enter your Name"))
    email = schema.TextLine(title=_(u"mail_address", default="E-Mail"),
                            required=True,
                            description=_(u"help_mail_address", default="Please enter your e-mail"))
    subject = schema.TextLine(title=_(u"label_subject", default="Subject"),
                              required=True,
                              description=_(u"help_subject", default="Please enter a Subject"))
    message = schema.Text(title=_(u"label_message", default="Message"),
                          required=True,
                          description=_(u"help_message", default="Please enter a Message"))


class ContactForm(form.Form):
    label = _(u"label_send_feedback", default=u"Send Feedback")
    fields = field.Fields(IContactForm)
    # don't use context to get widget data
    ignoreContext = True

    @button.buttonAndHandler(_(u'Send Mail'))
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            return

        message = data.get('message')
        email = data.get('email')
        subject = data.get('subject')
        sender = data.get('sender')
        self.send_feedback(email, subject, message, sender)
        msg = _(u'info_email_sent', default=u'The email was sent.')
        IStatusMessage(self.request).addStatusMessage(msg, type='info')
        return self.redirect()

    @button.buttonAndHandler(_(u'button_cancel', default=u'Cancel'))
    def handle_cancel(self, action):
        return self.redirect()

    def redirect(self):
        """Redirect back
        """
        url = self.context.absolute_url()
        return self.request.RESPONSE.redirect(url)

    def send_feedback(self, recipient, subject, message, sender):
        """Send a feedback email to the email address defined in subsite.
        """
        mh = getToolByName(self.context, 'MailHost')
        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        nav_root = None
        if not '/'.join(portal.getPhysicalPath()) == getNavigationRoot(self.context):
            nav_root = self.context.restrictedTraverse(
                getNavigationRoot(self.context))
        if nav_root:
            site_title = nav_root.Title()
            site_url = nav_root.absolute_url()
        else:
            site_title = portal.Title()
            site_url = portal.absolute_url()

        text = translate(
            u'feedback_mail_text',
            domain='ftw.subsite',
            default='${sender} sends you a message from your site ${site_title}(${site_url}):\n${msg}',
            context=self.request,
            mapping={'sender': "%s (%s)" % (sender, recipient),
                     'msg': message,
                     'site_title': site_title,
                     'site_url': site_url})

        # create the message root with from, to, and subject headers
        msg = MIMEText(text.encode('windows-1252'), 'plain', 'windows-1252')
        msg['Subject'] = Header(subject, 'windows-1252')

        default_from_name = portal.getProperty('email_from_name', '')
        default_from_email = portal.getProperty('email_from_address', '')

        if nav_root:
            email_from_name = nav_root.from_name or default_from_name
            email_from_email = nav_root.from_email or default_from_email
        else:
            email_from_name = default_from_name
            email_from_email = default_from_email
        msg['From'] = "%s <%s>" % (
            email_from_name,
            email_from_email
        )

        msg['reply-to'] = "%s <%s>" % (sender, recipient)

        if IS_PLONE_5:
            from plone import api
            reg = api.portal.get_tool('portal_registry')
            from_email_field = reg._records.get('plone.email_from_address')
            if not email_from_email and not from_email_field.value:
                # especially for testing reasons we need to set a value here
                # if none is registered on the page in plone5
                from_email_field._set_value('site@nohost.com')
                email_from_email = from_email_field.value
        # send the message
        mh.send(msg, mto=email_from_email,
                mfrom=[email_from_email])


class AddressesValidator(SimpleFieldValidator):
    """Validator for validating the e-mail addresses field
    """

    MAIL_EXPRESSION = r"^(\w&.%#$&'\*+-/=?^_`{}|~]+!)*[\w&.%#$&'\*+-/=" +\
        "?^_`{}|~]+@(([0-9a-z]([0-9a-z-]*[0-9a-z])?" +\
        "\.)+[a-z]{2,6}|([0-9]{1,3}\.){3}[0-9]{1,3})$"

    def __init__(self, *args, **kwargs):
        super(AddressesValidator, self).__init__(*args, **kwargs)
        self.email_expression = re.compile(AddressesValidator.MAIL_EXPRESSION,
                                           re.IGNORECASE)

    def validate(self, value):
        """Validates the `value`, expects a list of carriage-return-separated
        email addresses.
        """
        super(AddressesValidator, self).validate(value)
        if value:
            address = value.strip()
            self._validate_addresses(address)

    def _validate_addresses(self, address):
        """E-Mail address validation
        """
        address = address.strip()
        if not self.email_expression.match(address):
            msg = _(u'error_invalid_addresses',
                    default=u'Your E-mail address is not valid.')
            raise Invalid(msg)


WidgetValidatorDiscriminators(AddressesValidator,
                              field=IContactForm['email'])
provideAdapter(AddressesValidator)



from ftw.builder import Builder
from ftw.builder import builder_registry
from ftw.builder import create
from ftw.builder import session
from ftw.builder.dexterity import DexterityBuilder
from ftw.simplelayout.contenttypes.behaviors import ITeaser
from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.interfaces import IPageConfiguration
from plone.app.contenttypes.migration.migration import ATCTFolderMigrator
from plone.app.textfield.value import RichTextValue
from plone.app.uuid.utils import uuidToObject
from plone.i18n.normalizer.interfaces import IFileNameNormalizer
from plone.namedfile.file import NamedBlobImage
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.utils import safe_unicode
from Products.contentmigration.basemigrator.walker import CatalogWalker
from z3c.relationfield import RelationValue
from zope.annotation import IAnnotations
from zope.component import getUtility
from zope.intid.interfaces import IIntIds


class BuilderSession(object):
    auto_commit = False


session.current_session = BuilderSession()


class TextBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.simplelayout.TextBlock'

try:
    # Plone reload support
    builder_registry.register('sl textblock', TextBlockBuilder)
except ValueError:
    pass


class FtwSubsiteMigrator(ATCTFolderMigrator):

    src_portal_type = 'Subsite'
    src_meta_type = 'Subsite'
    dst_portal_type = 'ftw.subsite.Subsite'
    dst_meta_type = None  # not used

    def beforeChange_language_reference(self):
        language_reference_uids = self.old.getRawLanguage_references()
        if language_reference_uids:
            setattr(self.old,
                    '_language_reference_uids',
                    language_reference_uids)

    def migrate_prepare_page_state(self):
        config = IPageConfiguration(self.new)

        page_config = {
            "default": [
                {"cols": [
                    {"blocks": []},
                    {"blocks": []},
                    {"blocks": []},
                    {"blocks": []},
                ]},
                {"cols": [
                    {"blocks": []},
                    {"blocks": []}]
                 }]}
        config.store(page_config)

    def migrate_subsite_fields(self):
        self.new.additional_css = self.old.getAdditional_css()
        self.new.link_site_in_languagechooser = \
            self.old.getField('linkSiteInLanguagechooser').get(self.old)
        self.new.force_language = self.old.getForcelanguage()
        self.new.from_name = self.old.getFromName()
        self.new.from_email = self.old.getFromEmail()

        old_logo = self.old.getField('logo').get(self.old)
        if old_logo.data == '':
            return
        filename = safe_unicode(old_logo.filename)
        normalizer = getUtility(IFileNameNormalizer).normalize
        namedblobimage = NamedBlobImage(data=old_logo.data,
                                        filename=normalizer(filename).decode('utf-8'))
        self.new.logo = namedblobimage

    def migrate_language_references(self):
        # Check restore_language_reference method in migration.py for internal
        # references.
        if hasattr(self.old, '_language_reference_uids'):
            setattr(self.new,
                    '_language_reference_uids',
                    self.old._language_reference_uids)

    def migrate_static_text_portlets_to_textblock(self):
        annotations = IAnnotations(self.old)
        portlets = annotations.get('plone.portlets.contextassignments', None)
        if portlets is None:
            return

        managers = ['ftw.subsite.front1', 'ftw.subsite.front2',
                    'ftw.subsite.front3', 'ftw.subsite.front4',
                    'ftw.subsite.front5', 'ftw.subsite.front6',
                    'ftw.subsite.front7']

        for manager in managers:
            for portlet in portlets.get(manager, {}).values():
                if portlet.__module__ == 'ftw.subsite.portlets.teaserportlet':
                    self.create_block(portlet, manager, 'teaser')
                elif portlet.__module__ == 'plone.portlet.static.static':
                    self.create_block(portlet, manager, 'static')
                else:
                    pass

    def create_block(self, portlet, manager, type_):

        config = IPageConfiguration(self.new)
        normalizer = getUtility(IFileNameNormalizer).normalize
        if type_ == 'teaser':
            block = create(Builder('sl textblock')
                           .within(self.new)
                           .titled(portlet.teasertitle)
                           .having(text=RichTextValue(portlet.teaserdesc),
                                   image=NamedBlobImage(
                                       filename=normalizer(
                                           portlet.image.filename).decode('utf-8'),
                                       data=portlet.image.data)))

            blockconfig = IBlockConfiguration(block)
            blockconfigdata = blockconfig.load()
            blockconfigdata['scale'] = 'large'
            blockconfigdata['imagefloat'] = 'no-float'
            blockconfig.store(blockconfigdata)

            if portlet.internal_target:
                teaser = ITeaser(block)
                target = uuidToObject(portlet.internal_target)
                if target:
                    intids = getUtility(IIntIds)
                    teaser.internal_link = RelationValue(intids.getId(target))

        elif type_ == 'static':
            block = create(Builder('sl textblock')
                           .within(self.new)
                           .titled(portlet.header)
                           .having(text=RichTextValue(portlet.text)))
        else:
            return

        uid = IUUID(block)

        page_state = config.load()
        if manager == 'ftw.subsite.front1':
            page_state['default'][0]['cols'][0]['blocks'].append({'uid': uid})
        elif manager == 'ftw.subsite.front2':
            page_state['default'][0]['cols'][1]['blocks'].append({'uid': uid})
        elif manager == 'ftw.subsite.front3':
            page_state['default'][0]['cols'][2]['blocks'].append({'uid': uid})
        elif manager == 'ftw.subsite.front4':
            page_state['default'][0]['cols'][3]['blocks'].append({'uid': uid})
        elif manager == 'ftw.subsite.front5':
            page_state['default'][1]['cols'][0]['blocks'].append({'uid': uid})
        elif manager == 'ftw.subsite.front6':
            page_state['default'][1]['cols'][1]['blocks'].append({'uid': uid})
        # Don't know where manager 7 belongs
        elif manager == 'ftw.subsite.front7':
            page_state['default'][1]['cols'][0]['blocks'].append({'uid': uid})

        config.store(page_state)


def migrate(portal, migrator):
    """return a CatalogWalker instance in order
    to have its output after migration"""
    walker = CatalogWalker(portal, migrator)()
    return walker


def subsite_migrator(portal):
    return migrate(portal, FtwSubsiteMigrator)



"""
Copied from elyself.portlets.image
the image portlet is not working proper under plone 3.0.6
"""
from Acquisition import aq_inner
from plone.namedfile.file import FileChunk
from zope.publisher.browser import BrowserView


class ImageView(BrowserView):
    '''
    View the image field of the image portlet. We steal header details
    from zope.app.file.browser.file and adapt it to use the dublin
    core implementation that the Image object here has.
    '''

    def __call__(self):
        context = aq_inner(self.context)
        image = context.image

        data = image.data
        if isinstance(image.data, FileChunk):
            data = str(data)

        # cache for one day.
        self.request.response.setHeader('Cache-Control', 'max-age=86400')
        self.request.response.setHeader('Content-Type', image.contentType)
        self.request.response.write(data)



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.testing import FTW_SUBSITE_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import statusmessages
from unittest2 import TestCase
import email
import transaction


class TestContactFrom(TestCase):

    layer = FTW_SUBSITE_FUNCTIONAL_TESTING

    def setUp(self):

        self.portal = self.layer['portal']
        self.subsite = create(Builder('subsite')
                              .titled(u'Subsite')
                              .having(from_email=u'blubber@blubb.ch',
                                      from_name=u'Subsite'))

    @browsing
    def test_contact_everything_ok(self, browser):
        browser.login().visit(self.subsite, view='contact-info')
        browser.fill({'form.widgets.sender': u'hans peter',
                      'form.widgets.email': u'test@test.com',
                      'form.widgets.subject': u'Testsubject',
                      'form.widgets.message': u'Lorem ipsum dolor sit amet'}
                     ).submit()

        mh = self.portal.MailHost
        self.assertEqual(len(mh.messages), 1)

        msg = email.message_from_string(mh.messages[0])
        self.assertEqual(msg.get('From'), 'blubber@blubb.ch')
        self.assertEqual(msg.get('reply-to'), u'hans peter <test@test.com>')
        sub = msg.get('Subject')
        self.assertEqual(sub.encode('utf8'), 'Testsubject')
        self.assertEqual(
            ('hans peter (test@test.com) sends you a message from your site '
             'Subsite(http:=\n//nohost/plone/subsite):\n'
             'Lorem ipsum dolor sit amet'),
            msg.get_payload())

    @browsing
    def test_contact_cancel(self, browser):
        browser.login().visit(self.subsite, view='contact-info')
        browser.find_button_by_label('Cancel').click()
        self.assertEqual(browser.url, self.subsite.absolute_url())

    @browsing
    def test_contact_portal(self, browser):
        self.portal.setTitle('Test')
        self.portal.manage_changeProperties(
            email_from_address='site@nohost.com')
        self.portal.manage_changeProperties(email_from_name='Ploneroot')
        transaction.commit()

        browser.login().visit(self.portal, view='@@contact-info')

        browser.fill({'form.widgets.sender': u'hans peter',
                      'form.widgets.email': u'test@test.com',
                      'form.widgets.subject': u'Testsubject',
                      'form.widgets.message': u'Lorem ipsum dolor sit amet'}
                     ).submit()

        mh = self.portal.MailHost
        self.assertEqual(len(mh.messages), 1)

        msg = email.message_from_string(mh.messages[0])
        self.assertEqual(msg.get('From'), 'site@nohost.com')
        self.assertEqual(msg.get('reply-to'), u'hans peter <test@test.com>')

        sub = msg.get('Subject')
        self.assertEqual(sub.encode('utf8'), 'Testsubject')
        self.assertEqual(
            ('hans peter (test@test.com) sends you a message from your site '
             'Test(http://n=\nohost/plone):\n'
             'Lorem ipsum dolor sit amet'),
            msg.get_payload())

    @browsing
    def test_contact_invalid_email(self, browser):
        browser.login().visit(self.subsite, view='contact-info')
        browser.fill({'form.widgets.sender': u'hans peter',
                      'form.widgets.email': u'test@test',
                      'form.widgets.subject': u'Testsubject',
                      'form.widgets.message': u'Lorem ipsum dolor sit amet'}
                     ).submit()
        self.assertIn(
            '<div class="error">Your E-mail address is not valid.</div>',
            browser.contents)



from Acquisition import aq_inner
from ftw.subsite.utils import get_nav_root
from plone.app.layout.navigation.interfaces import INavtreeStrategy
from plone.app.layout.navigation.navtree import buildFolderTree
from Products.CMFPlone.browser import navigation
from Products.CMFPlone.browser import navtree
from zope.component import getMultiAdapter


class SubsiteSitemapQueryBuilder(navtree.SitemapQueryBuilder):

    def __init__(self, context):
        super(SubsiteSitemapQueryBuilder, self).__init__(context)

        root = get_nav_root(context)
        self.query['path']['query'] = '/'.join(root.getPhysicalPath())


class SubsiteCatalogSiteMap(navigation.CatalogSiteMap):

    def siteMap(self):
        context = aq_inner(self.context)

        queryBuilder = SubsiteSitemapQueryBuilder(context)
        query = queryBuilder()

        strategy = getMultiAdapter((context, self), INavtreeStrategy)

        return buildFolderTree(context, obj=context,
                               query=query, strategy=strategy)



from Acquisition import aq_inner, aq_parent
from DateTime import DateTime
from ftw.subsite import _
from plone.app.portlets.portlets import base
from plone.app.uuid.utils import uuidToObject
from plone.app.z3cform.wysiwyg.widget import WysiwygFieldWidget
from plone.formwidget.contenttree import ContentTreeFieldWidget
from plone.formwidget.contenttree import UUIDSourceBinder
from plone.namedfile.field import NamedImage
from plone.portlets.interfaces import IPortletDataProvider
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import form, button, field
from z3c.form.interfaces import INPUT_MODE
from zope import schema
from zope.component import getMultiAdapter
from zope.interface import implements


class ITeaserPortlet(IPortletDataProvider):
    """A portlet which can some given Infos
    """

    teasertitle = schema.TextLine(title=_(u'Title'),
                                  description=u'',
                                  required=True)

    teaserdesc = schema.Text(title=_(u'Description'),
                             description=u'',
                             required=True)

    internal_target = schema.Choice(title=_(u"Internal Target"),
                                    description=_(u"Find an internal target \
 for this image to link to"),
                                   required=False,
                                   source=UUIDSourceBinder({})
                                    )
    image = NamedImage(title=_(u'Image'),
                        description=u'',
                        required=True)

    imagetitle = schema.TextLine(
        title=_(
            u'label_imagetitle',
            default=u'Image Title',
        ),
        description=_(
            u'help_imagetitle',
            default=u'The provided image title will be used as alt-text.',
        ),
        required=True,
    )


class Assignment(base.Assignment):
    implements(ITeaserPortlet)

    def __init__(self, assignment_context_path=None, teasertitle='',
                 teaserdesc='', image=None, _image=None,
                 imagetitle='', internal_target=None, image_timestamp=None):

        # ftw.publisher support
        if _image:
            image = _image

        self.assignment_context_path = assignment_context_path
        self.teasertitle = teasertitle
        self.teaserdesc = teaserdesc
        self._image = image
        self.imagetitle = imagetitle
        self.internal_target = internal_target
        self.image_timestamp = image_timestamp or DateTime()

    @property
    def image(self):
        return self._image

    @image.setter
    def image(self, image):
        self.image_timestamp = DateTime()
        self._image = image

    @property
    def title(self):
        return self.teasertitle


class Renderer(base.Renderer):

    render = ViewPageTemplateFile('teaserportlet.pt')

    def getTeaserTitle(self):
        return self.data.teasertitle

    def getTeaserDesc(self):
        return self.data.teaserdesc

    @property
    def internal_url(self):
        object_path = self.data.internal_target
        internal_obj = uuidToObject(object_path)
        if internal_obj:
            return internal_obj.absolute_url()
        else:
            return ''

    @property
    def image_tag(self):
        state = getMultiAdapter((self.context, self.request),
                                name="plone_portal_state")
        portal = state.portal()

        assignments = portal.unrestrictedTraverse(
            self.data.assignment_context_path)
        assignment_url = assignments.absolute_url()
        modified = self.data.image_timestamp
        return u"<img src='{0}/{1}/@@image?_={2}' alt='{3}'/>".format(
            assignment_url,
            self.data.__name__,
            modified.millis(),
            getattr(self.data, 'imagetitle', '')
        )


class AddForm(form.AddForm):
    fields = field.Fields(ITeaserPortlet)

    label = _(u"Add Teaser Portlet")
    description = _(u"Shows the given infos on the front-page.")

    def __init__(self, context, request):
        super(AddForm, self).__init__(context, request)
        self._finishedAdd = None
        self.status = None

    def updateWidgets(self):
        self.fields['teaserdesc'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
        self.fields['internal_target'].widgetFactory = ContentTreeFieldWidget

        super(AddForm, self).updateWidgets()

    def nextURL(self):
        editview = aq_parent(aq_inner(self.context))
        context = aq_parent(aq_inner(editview))
        url = str(getMultiAdapter((context, self.request),
                                  name=u"absolute_url"))
        return url

    @button.buttonAndHandler(_(u"label_save", default=u"Save"), name='add')
    def handleAdd(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            # mark only as finished if we get the new object
            self._finishedAdd = True

    @button.buttonAndHandler(_(u"label_cancel", default=u"Cancel"),
                             name='cancel_add')
    def handleCancel(self, action):
        nextURL = self.nextURL()
        self.request.response.redirect(nextURL)

    def add(self, obj):
        ob = self.context.add(obj)
        self._finishedAdd = True
        return ob

    def create(self, data):

        portal_path_l = len(
            self.context.portal_url.getPortalObject().getPhysicalPath())
        assignment_context_path = '/'.join(
            self.context.__parent__.getPhysicalPath()[portal_path_l:])

        return Assignment(
            assignment_context_path=assignment_context_path, **data)


class EditForm(form.EditForm):
    fields = field.Fields(ITeaserPortlet)

    label = _(u"Edit Teaser Portlet")
    description = _(u"Shows the given infos on the front-page.")

    def updateWidgets(self):
        self.fields['teaserdesc'].widgetFactory[INPUT_MODE] = WysiwygFieldWidget
        self.fields['internal_target'].widgetFactory = ContentTreeFieldWidget
        super(EditForm, self).updateWidgets()



from ftw.builder import Builder
from ftw.builder import create
from ftw.subsite.tests.base import IntegrationTestCase
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds


def introduce_language_subsites(*subsites):
    intids = getUtility(IIntIds)

    for subsite in subsites:
        ids = [intids.getId(obj) for obj in subsites]
        ids.remove(intids.getId(subsite))
        subsite.language_references = [RelationValue(id_) for id_ in ids]


class TestLanguagesView(IntegrationTestCase):

    def setUp(self):
        self.portal = self.layer['portal']

    def test_available_when_subsites_language_referenced(self):
        german = create(Builder('subsite')
                        .titled(u'Subsite DE')
                        .with_language('de'))
        french = create(Builder('subsite')
                        .titled(u'Subsite FR')
                        .with_language('fr'))
        introduce_language_subsites(german, french)
        self.assertTrue(
            german.restrictedTraverse('@@subsite-languages').available())

    def test_not_available_unless_languages_hooked_up(self):
        german = create(Builder('subsite')
                        .titled(u'Subsite DE')
                        .with_language('de'))
        self.assertFalse(
            german.restrictedTraverse('@@subsite-languages').available())

    def test_available_on_site_root_when_subsites_hooked_up(self):
        create(Builder('subsite')
               .titled(u'Subsite DE')
               .with_language('de')
               .having(link_site_in_languagechooser=True))

        self.assertTrue(
            self.portal.restrictedTraverse('@@subsite-languages').available())

    def test_not_available_on_site_root_when_not_hooked_up(self):
        self.assertFalse(
            self.portal.restrictedTraverse('@@subsite-languages').available())



from setuptools import setup, find_packages
import os

version = '2.5.0'

tests_require = [
    'collective.mockmailhost',
    'ftw.builder',
    'ftw.testing',
    'plone.app.contenttypes',
    'plone.app.portlets',
    'plone.app.testing',
    'pyquery',
    'ftw.testbrowser',
    'ftw.builder',
    'ftw.mobile',
    ]

extras_require = {
    'tests': tests_require,
    'test': tests_require,
    'plone4': [
        'ftw.simplelayout [plone4]',
    ],
}

setup(name='ftw.subsite',
      version=version,
      description="",
      long_description=open('README.rst').read() + '\n' +
      open(os.path.join('docs', 'HISTORY.txt')).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers

      classifiers=[
          'Framework :: Plone',
          'Framework :: Plone :: 4.3',
          'Framework :: Plone :: 5.1',
          'Programming Language :: Python',
          'Topic :: Software Development :: Libraries :: Python Modules',
      ],

      keywords='ftw subsite',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.subsite',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
          'ftw.upgrade >= 2.0.0',
          'plone.formwidget.contenttree',
          'plone.formwidget.namedfile',
          'plone.namedfile',
          'setuptools',
          'plone.app.dexterity',
          'plone.directives.form',
          'plone.app.relationfield',
          'plone.autoform',
          'plone.behavior',
          'ftw.simplelayout [contenttypes]',
          'plone.app.contenttypes',
          'plone.app.event',
          'ftw.builder',
          'ftw.theming >= 2.0.0',
          'Plone',
          'ftw.referencewidget',
      ],

      tests_require=tests_require,
      extras_require=extras_require,

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from Products.CMFCore.interfaces._content import IContentish
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api
from plone.app.layout.navigation.root import getNavigationRoot
from plone.app.layout.viewlets.common import LogoViewlet
import pkg_resources

IS_PLONE_5 = pkg_resources.get_distribution('Products.CMFPlone').version >= '5'


class SubsiteLogoViewlet(LogoViewlet):
    index = ViewPageTemplateFile('subsitelogoviewlet.pt')

    def __init__(self, context, request, view, manager=None):
        super(SubsiteLogoViewlet, self).__init__(
            context, request, view, manager)
        self.navigation_root_url = None
        self.logo_tag = None
        self.title = None
        self.is_subsitelogo = False

    def update(self):
        super(SubsiteLogoViewlet, self).update()

        navroot = api.portal.get_navigation_root(self.context)
        is_subsite = bool(navroot.portal_type == 'ftw.subsite.Subsite')

        self.navigation_root_url = self.portal_state.navigation_root_url()

        subsite_logo = getattr(navroot, 'logo', None)
        subsite_logo_alt_text = getattr(navroot, 'logo_alt_text', None)

        if is_subsite and subsite_logo and subsite_logo.data:
            # we are in a subsite
            context = self.context
            if not IContentish.providedBy(context):
                context = context.aq_parent
            navigation_root_path = getNavigationRoot(context)
            scale = navroot.restrictedTraverse(
                navigation_root_path + '/@@images')

            self.logo_tag = scale.scale('logo', scale="logo").tag(
                alt=subsite_logo_alt_text, title=None)
            self.title = self.context.restrictedTraverse(
                getNavigationRoot(self.context)).Title()
            self.is_subsitelogo = True
        else:
            # standard plone logo
            if not IS_PLONE_5:
                portal = api.portal.get()
                logoName = navroot.restrictedTraverse('base_properties').logoName
                logo_alt_text = portal.getProperty('logo_alt_text', '')
                self.logo_tag = portal.restrictedTraverse(logoName).tag(
                    alt=logo_alt_text, title='')
                self.title = self.portal_state.portal_title()

            if IS_PLONE_5:
                portal = api.portal.get()
                self.title = self.portal_state.portal_title()
                portal_path = "/".join(portal.getPhysicalPath())
                logo_path = '{}/logo.png'.format(portal_path)
                logo_alt_text = portal.getProperty('logo_alt_text', self.title)
                self.logo_tag = portal.restrictedTraverse(logo_path).tag(
                    alt=logo_alt_text, title='')
                self.is_subsitelogo = False



from ftw.referencewidget.widget import ReferenceBrowserWidget
from ftw.subsite import _
from ftw.subsite.interfaces import ISubsite
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Container
from plone.directives.form import widget
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import directives
from plone.supermodel import model
from z3c.relationfield.schema import RelationList
from zope import schema
from zope.interface import alsoProvides
from zope.interface import implements


class ISubsiteSchema(model.Schema):
    """The Subsite schema"""

    directives.fieldset(
        'subsite',
        label=_(u'Subsite'),
        fields=('logo', 'logo_alt_text', 'additional_css',
                'language_references', 'link_site_in_languagechooser',
                'force_language')
    )

    logo = NamedBlobImage(
        title=_(u'label_logo', default=u'Logo'),
        description=_(u'help_logo', default=u''),
        required=False
    )

    logo_alt_text = schema.TextLine(
        title=_(u'label_logo_alt_text',
                default=u'Alternative text of the logo'),
        required=False,
        default=u'',
    )

    additional_css = schema.Text(
        title=_(u'label_additional_css', default=u'Additional CSS'),
        description=_(u'help_additional_css', default=u''),
        required=False,
        missing_value=''
    )
    widget('language_references', ReferenceBrowserWidget,
           override=True, selectable=["ftw.subsite.Subsite"])
    language_references = RelationList(
        title=_(u'label_language_references', default=u'Languages'),
        default=[],
        missing_value=[],
        required=False,
        description=_(u'help_language_references',
                      default=_(u'The language switch will only be '
                                'displayed, if the chosen Subsite(s)'
                                'has a value in the forcelangage '
                                'field'))
    )

    link_site_in_languagechooser = schema.Bool(
        title=_(u'label_link_site_in_languagechooser',
                default=u'Link Plone Site in language chooser'),
        default=False,
        required=False)

    force_language = schema.Choice(
        title=_(u'label_forcelanguage', default=u'Subsite language'),
        description=_(u'help_forcelanguage',
                      default=u'The Subsite and it\'s content will be '
                      'delivered in the choosen language'),
        vocabulary='ftw.subsites.languages',
        required=False
    )

    from_name = schema.TextLine(
        title=_(u'label_fromname', default=u'Email Sendername'),
        description=_(u'help_fromname', default=u''),
        required=False)

    from_email = schema.TextLine(
        title=_(u'label_fromemail', default=u'Email Senderaddress'),
        description=_(u'help_fromname', default=u''),
        required=False)

alsoProvides(ISubsiteSchema, IFormFieldProvider)


class Subsite(Container):
    implements(ISubsite, INavigationRoot)



from ftw.subsite.interfaces import IFtwSubsiteLayer
from ftw.subsite.interfaces import ISubsite
from ftw.subsite.utils import find_context
from ftw.subsite.utils import get_nav_root
from Products.CMFCore.interfaces import IContentish
from Products.CMFCore.interfaces import IFolderish
from zope.i18n.interfaces import INegotiator
from zope.i18n.negotiator import negotiator as base_negotiator
from zope.interface import implements


class Negotiator(object):
    """A language negotiator whicht gets the language from an attribute stored
       on the subsite
    """
    implements(INegotiator)

    def getLanguage(self, langs, env):

        subsite_language = get_subsite_language(env)
        return subsite_language or base_negotiator.getLanguage(langs, env)

negotiator = Negotiator()


def get_subsite_language(request):
    """Returns None, or a language code."""
    if not IFtwSubsiteLayer.providedBy(request):
        return None

    # Get current published object
    obj = find_context(request)
    # Filter out CSS/JS and other non contentish objects
    # IFolderish check includes site root
    if not (IContentish.providedBy(obj) or IFolderish.providedBy(obj)):
        return None

    nav_root = get_nav_root(obj)

    if ISubsite.providedBy(nav_root):
        # Get language stored on Subsite
        language = nav_root.force_language
        if language:
            return language
        else:
            return None

    else:
        return None





def find_context(request):
    """Find the context from the request

    http://stackoverflow.com/questions/10489544/getting-published-content-
    item-out-of-requestpublished-in-plone
    """
    published = request.get('PUBLISHED', None)
    context = getattr(published, '__parent__', None)
    if context is None:
        context = request.PARENTS[0]
    return context


def get_nav_root(context):
    """Returns the navigation root"""

    plone_state = context.restrictedTraverse('@@plone_portal_state')
    return plone_state.navigation_root()
